@@ -1,46 +1,42 @@
-use crate::{
-    PKEY_Device_DeviceDesc, PKEY_Device_FriendlyName,
-    Windows::Win32::Foundation::{BOOL, HANDLE, PSTR, PWSTR, S_OK},
-    Windows::Win32::Media::Audio::CoreAudio::{
-        eCapture, eConsole, eRender, AudioSessionDisconnectReason, AudioSessionState,
-        AudioSessionStateActive, AudioSessionStateExpired, AudioSessionStateInactive,
-        DisconnectReasonDeviceRemoval, DisconnectReasonExclusiveModeOverride,
-        DisconnectReasonFormatChanged, DisconnectReasonServerShutdown,
-        DisconnectReasonSessionDisconnected, DisconnectReasonSessionLogoff, IAudioCaptureClient,
-        IAudioClient, IAudioRenderClient, IAudioSessionControl, IAudioSessionEvents,
-        IMMDevice, IMMDeviceCollection, IMMDeviceEnumerator,
-        MMDeviceEnumerator, AUDCLNT_SHAREMODE_EXCLUSIVE, AUDCLNT_SHAREMODE_SHARED,
-        AUDCLNT_STREAMFLAGS_AUTOCONVERTPCM, AUDCLNT_STREAMFLAGS_EVENTCALLBACK,
-        AUDCLNT_STREAMFLAGS_LOOPBACK, AUDCLNT_STREAMFLAGS_SRC_DEFAULT_QUALITY, DEVICE_STATE_ACTIVE,
-        WAVE_FORMAT_EXTENSIBLE,
+use crate::Windows;
+use crate::Windows::{
+    Win32::Foundation::{HANDLE, PSTR},
+    Win32::Media::Audio::CoreAudio::{
+        eCapture, eConsole, eRender, AudioSessionStateActive, AudioSessionStateExpired,
+        AudioSessionStateInactive, IAudioCaptureClient, IAudioClient, IAudioRenderClient,
+        IAudioSessionControl, IAudioSessionEvents, IMMDevice, IMMDeviceCollection,
+        IMMDeviceEnumerator, MMDeviceEnumerator, AUDCLNT_SHAREMODE_EXCLUSIVE,
+        AUDCLNT_SHAREMODE_SHARED, AUDCLNT_STREAMFLAGS_AUTOCONVERTPCM,
+        AUDCLNT_STREAMFLAGS_EVENTCALLBACK, AUDCLNT_STREAMFLAGS_LOOPBACK,
+        AUDCLNT_STREAMFLAGS_SRC_DEFAULT_QUALITY, DEVICE_STATE_ACTIVE, WAVE_FORMAT_EXTENSIBLE,
     },
-    Windows::Win32::Media::Multimedia::{
-        KSDATAFORMAT_SUBTYPE_IEEE_FLOAT, KSDATAFORMAT_SUBTYPE_PCM, WAVEFORMATEX,
-        WAVEFORMATEXTENSIBLE, WAVEFORMATEXTENSIBLE_0, WAVE_FORMAT_IEEE_FLOAT, WAVE_FORMAT_PCM,
-    },
-    Windows::Win32::Storage::StructuredStorage::STGM_READ,
-    Windows::Win32::System::Com::CLSCTX_ALL,
-    Windows::Win32::System::Com::{
+    Win32::Media::Multimedia::{WAVEFORMATEX, WAVEFORMATEXTENSIBLE},
+    Win32::System::Com::StructuredStorage::STGM_READ,
+    Win32::System::Com::CLSCTX_ALL,
+    Win32::System::Com::{
         CoCreateInstance, CoInitializeEx, COINIT_APARTMENTTHREADED, COINIT_MULTITHREADED,
     },
-    Windows::Win32::System::PropertiesSystem::PropVariantToStringAlloc,
-    Windows::Win32::System::Threading::{CreateEventA, WaitForSingleObject, WAIT_OBJECT_0},
+    Win32::System::PropertiesSystem::PropVariantToStringAlloc,
+    Win32::System::Threading::{CreateEventA, WaitForSingleObject},
 };
 use std::collections::VecDeque;
 use std::error;
 use std::fmt;
 use std::mem;
 use std::ptr;
-use std::slice;
 use std::rc::Weak;
+use std::slice;
 use widestring::U16CString;
-use windows::Guid;
-use windows::Interface;
-use windows::HRESULT;
-use windows::implement;
-use crate::Windows;
+use windows::runtime::Interface;
+use Windows::Win32::System::SystemServices::{
+    DEVPKEY_Device_DeviceDesc, DEVPKEY_Device_FriendlyName,
+};
+use Windows::Win32::System::Threading::WAIT_OBJECT_0;
 
-type WasapiRes<T> = Result<T, Box<dyn error::Error>>;
+use crate::WaveFormat;
+use crate::{AudioSessionEvents, EventCallbacks};
+
+pub(crate) type WasapiRes<T> = Result<T, Box<dyn error::Error>>;
 
 /// Error returned by the Wasapi crate.
 #[derive(Debug)]
@@ -69,12 +65,12 @@ impl WasapiError {
 }
 
 /// Initializes COM for use by the calling thread for the multi-threaded apartment (MTA).
-pub fn initialize_mta() -> Result<(), windows::Error> {
+pub fn initialize_mta() -> Result<(), windows::runtime::Error> {
     unsafe { CoInitializeEx(std::ptr::null_mut(), COINIT_MULTITHREADED) }
 }
 
 /// Initializes COM for use by the calling thread for a single-threaded apartment (STA).
-pub fn initialize_sta() -> Result<(), windows::Error> {
+pub fn initialize_sta() -> Result<(), windows::runtime::Error> {
     unsafe { CoInitializeEx(std::ptr::null_mut(), COINIT_APARTMENTTHREADED) }
 }
 
@@ -109,19 +105,13 @@ pub fn get_default_device(direction: &Direction) -> WasapiRes<Device> {
     let enumerator: IMMDeviceEnumerator =
         unsafe { CoCreateInstance(&MMDeviceEnumerator, None, CLSCTX_ALL)? };
     let device = unsafe { enumerator.GetDefaultAudioEndpoint(dir, eConsole)? };
-    debug!("default device {:?}", device);
 
-    //match device {
-    //    Some(dev) => Ok(Device {
-    //        device: dev,
-    //        direction: direction.clone(),
-    //    }),
-    //    None => Err(WasapiError::new("Failed to get default device").into()),
-    //}
-    Ok(Device {
+    let dev = Device {
         device,
         direction: direction.clone(),
-    })
+    };
+    debug!("default device {:?}", dev.get_friendlyname());
+    Ok(dev)
 }
 
 /// Struct wrapping an IMMDeviceCollection.
@@ -211,7 +201,7 @@ impl Device {
     /// Read the FriendlyName of an IMMDevice
     pub fn get_friendlyname(&self) -> WasapiRes<String> {
         let store = unsafe { self.device.OpenPropertyStore(STGM_READ as u32)? };
-        let prop = unsafe { store.GetValue(&PKEY_Device_FriendlyName)? };
+        let prop = unsafe { store.GetValue(&DEVPKEY_Device_FriendlyName)? };
         let propstr = unsafe { PropVariantToStringAlloc(&prop)? };
         let wide_name = unsafe { U16CString::from_ptr_str(propstr.0) };
         let name = wide_name.to_string_lossy();
@@ -222,7 +212,7 @@ impl Device {
     /// Read the Description of an IMMDevice
     pub fn get_description(&self) -> WasapiRes<String> {
         let store = unsafe { self.device.OpenPropertyStore(STGM_READ as u32)? };
-        let prop = unsafe { store.GetValue(&PKEY_Device_DeviceDesc)? };
+        let prop = unsafe { store.GetValue(&DEVPKEY_Device_DeviceDesc)? };
         let propstr = unsafe { PropVariantToStringAlloc(&prop)? };
         let wide_desc = unsafe { U16CString::from_ptr_str(propstr.0) };
         let desc = wide_desc.to_string_lossy();
@@ -251,19 +241,17 @@ impl AudioClient {
     /// Get MixFormat of the device. This is the format the device uses in shared mode and should always be accepted.
     pub fn get_mixformat(&self) -> WasapiRes<WaveFormat> {
         let temp_fmt_ptr = unsafe { self.client.GetMixFormat()? };
-        let temp_fmt = unsafe {*temp_fmt_ptr};
-        let mix_format = if temp_fmt.cbSize == 22
-            && temp_fmt.wFormatTag as u32 == WAVE_FORMAT_EXTENSIBLE
-        {
-            unsafe {
-                WaveFormat {
-                    wave_fmt: (temp_fmt_ptr as *const _ as *const WAVEFORMATEXTENSIBLE)
-                        .read(),
+        let temp_fmt = unsafe { *temp_fmt_ptr };
+        let mix_format =
+            if temp_fmt.cbSize == 22 && temp_fmt.wFormatTag as u32 == WAVE_FORMAT_EXTENSIBLE {
+                unsafe {
+                    WaveFormat {
+                        wave_fmt: (temp_fmt_ptr as *const _ as *const WAVEFORMATEXTENSIBLE).read(),
+                    }
                 }
-            }
-        } else {
-            WaveFormat::from_waveformatex(temp_fmt)?
-        };
+            } else {
+                WaveFormat::from_waveformatex(temp_fmt)?
+            };
         Ok(mix_format)
     }
 
@@ -286,10 +274,8 @@ impl AudioClient {
             }
             ShareMode::Shared => {
                 let supported_format = unsafe {
-                    self.client.IsFormatSupported(
-                        AUDCLNT_SHAREMODE_SHARED,
-                        wave_fmt.as_waveformatex_ptr(),
-                    )
+                    self.client
+                        .IsFormatSupported(AUDCLNT_SHAREMODE_SHARED, wave_fmt.as_waveformatex_ptr())
                 }?;
                 // Check if we got a pointer to a WAVEFORMATEX structure.
                 if supported_format.is_null() {
@@ -374,7 +360,7 @@ impl AudioClient {
         Ok(())
     }
 
-    /// Create an return an event handle for an IAudioClient
+    /// Create and return an event handle for an IAudioClient
     pub fn set_get_eventhandle(&self) -> WasapiRes<Handle> {
         let h_event = unsafe { CreateEventA(std::ptr::null_mut(), false, false, PSTR::default()) };
         unsafe { self.client.SetEventHandle(h_event)? };
@@ -436,32 +422,47 @@ impl AudioClient {
 
     /// Get a rendering (playback) client
     pub fn get_audiorenderclient(&self) -> WasapiRes<AudioRenderClient> {
-        let renderclient: Option<IAudioRenderClient> = unsafe { self.client.GetService().ok() };
-        match renderclient {
-            Some(client) => Ok(AudioRenderClient { client }),
-            None => Err(WasapiError::new("Failed getting IAudioRenderClient").into()),
+        let mut renderclient_ptr = ptr::null_mut();
+        unsafe {
+            self.client
+                .GetService(&IAudioRenderClient::IID, &mut renderclient_ptr)?
+        };
+        if renderclient_ptr.is_null() {
+            return Err(WasapiError::new("Failed getting IAudioCaptureClient").into());
         }
+        let client = unsafe { mem::transmute(renderclient_ptr) };
+        Ok(AudioRenderClient { client })
     }
 
     /// Get a capture client
     pub fn get_audiocaptureclient(&self) -> WasapiRes<AudioCaptureClient> {
-        let renderclient: Option<IAudioCaptureClient> = unsafe { self.client.GetService().ok() };
-        match renderclient {
-            Some(client) => Ok(AudioCaptureClient {
-                client,
-                sharemode: self.sharemode.clone(),
-            }),
-            None => Err(WasapiError::new("Failed getting IAudioCaptureClient").into()),
+        let mut renderclient_ptr = ptr::null_mut();
+        unsafe {
+            self.client
+                .GetService(&IAudioCaptureClient::IID, &mut renderclient_ptr)?
+        };
+        if renderclient_ptr.is_null() {
+            return Err(WasapiError::new("Failed getting IAudioCaptureClient").into());
         }
+        let client = unsafe { mem::transmute(renderclient_ptr) };
+        Ok(AudioCaptureClient {
+            client,
+            sharemode: self.sharemode.clone(),
+        })
     }
 
     /// Get the AudioSessionControl
     pub fn get_audiosessioncontrol(&self) -> WasapiRes<AudioSessionControl> {
-        let sessioncontrol: Option<IAudioSessionControl> = unsafe { self.client.GetService().ok() };
-        match sessioncontrol {
-            Some(control) => Ok(AudioSessionControl { control }),
-            None => Err(WasapiError::new("Failed getting IAudioSessionControl").into()),
+        let mut sessioncontrol_ptr = ptr::null_mut();
+        unsafe {
+            self.client
+                .GetService(&IAudioSessionControl::IID, &mut sessioncontrol_ptr)?
+        };
+        if sessioncontrol_ptr.is_null() {
+            return Err(WasapiError::new("Failed getting IAudioSessionControl").into());
         }
+        let control = unsafe { mem::transmute(sessioncontrol_ptr) };
+        Ok(AudioSessionControl { control })
     }
 }
 
@@ -534,10 +535,7 @@ impl AudioRenderClient {
             )
             .into());
         }
-        let bufferptr = unsafe {
-            self.client
-                .GetBuffer(nbr_frames as u32)?
-        };
+        let bufferptr = unsafe { self.client.GetBuffer(nbr_frames as u32)? };
         let bufferslice = unsafe { slice::from_raw_parts_mut(bufferptr, nbr_bytes) };
         bufferslice.copy_from_slice(data);
         unsafe { self.client.ReleaseBuffer(nbr_frames as u32, 0)? };
@@ -561,10 +559,7 @@ impl AudioRenderClient {
             )
             .into());
         }
-        let bufferptr = unsafe {
-            self.client
-                .GetBuffer(nbr_frames as u32)?
-        };
+        let bufferptr = unsafe { self.client.GetBuffer(nbr_frames as u32)? };
         let bufferslice = unsafe { slice::from_raw_parts_mut(bufferptr, nbr_bytes) };
         for element in bufferslice.iter_mut() {
             *element = data.pop_front().unwrap();
@@ -609,6 +604,7 @@ impl AudioCaptureClient {
             )?
         };
         if nbr_frames_returned == 0 {
+            unsafe { self.client.ReleaseBuffer(nbr_frames_returned)? };
             return Ok(0);
         }
         if data_len_in_frames < nbr_frames_returned as usize {
@@ -675,403 +671,3 @@ impl Handle {
         Ok(())
     }
 }
-
-/// Struct wrapping a WAVEFORMATEXTENSIBLE format descriptor.
-#[derive(Clone)]
-pub struct WaveFormat {
-    wave_fmt: WAVEFORMATEXTENSIBLE,
-}
-
-impl fmt::Debug for WaveFormat {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        f.debug_struct("WaveFormat")
-            .field("nAvgBytesPerSec", &{ self.wave_fmt.Format.nAvgBytesPerSec })
-            .field("cbSize", &{ self.wave_fmt.Format.cbSize })
-            .field("nBlockAlign", &{ self.wave_fmt.Format.nBlockAlign })
-            .field("wBitsPerSample", &{ self.wave_fmt.Format.wBitsPerSample })
-            .field("nSamplesPerSec", &{ self.wave_fmt.Format.nSamplesPerSec })
-            .field("wFormatTag", &{ self.wave_fmt.Format.wFormatTag })
-            .field("wValidBitsPerSample", &unsafe {
-                self.wave_fmt.Samples.wValidBitsPerSample
-            })
-            .field("SubFormat", &{ self.wave_fmt.SubFormat })
-            .field("nChannel", &{ self.wave_fmt.Format.nChannels })
-            .field("dwChannelMask", &{ self.wave_fmt.dwChannelMask })
-            .finish()
-    }
-}
-
-impl WaveFormat {
-    /// Build a WAVEFORMATEXTENSIBLE struct for the given parameters
-    pub fn new(
-        storebits: usize,
-        validbits: usize,
-        sample_type: &SampleType,
-        samplerate: usize,
-        channels: usize,
-    ) -> Self {
-        let blockalign = channels * storebits / 8;
-        let byterate = samplerate * blockalign;
-
-        let wave_format = WAVEFORMATEX {
-            cbSize: 22,
-            nAvgBytesPerSec: byterate as u32,
-            nBlockAlign: blockalign as u16,
-            nChannels: channels as u16,
-            nSamplesPerSec: samplerate as u32,
-            wBitsPerSample: storebits as u16,
-            wFormatTag: WAVE_FORMAT_EXTENSIBLE as u16,
-        };
-        let sample = WAVEFORMATEXTENSIBLE_0 {
-            wValidBitsPerSample: validbits as u16,
-        };
-        let subformat = match sample_type {
-            SampleType::Float => KSDATAFORMAT_SUBTYPE_IEEE_FLOAT,
-            SampleType::Int => KSDATAFORMAT_SUBTYPE_PCM,
-        };
-        let mut mask = 0;
-        for n in 0..channels {
-            mask += 1 << n;
-        }
-        let wave_fmt = WAVEFORMATEXTENSIBLE {
-            Format: wave_format,
-            Samples: sample,
-            SubFormat: subformat,
-            dwChannelMask: mask,
-        };
-        WaveFormat { wave_fmt }
-    }
-
-    /// Create from a WAVEFORMATEX structure
-    pub fn from_waveformatex(wavefmt: WAVEFORMATEX) -> WasapiRes<Self> {
-        let validbits = wavefmt.wBitsPerSample as usize;
-        let blockalign = wavefmt.nBlockAlign as usize;
-        let samplerate = wavefmt.nSamplesPerSec as usize;
-        let formattag = wavefmt.wFormatTag;
-        let channels = wavefmt.nChannels as usize;
-        let sample_type = match formattag as u32 {
-            WAVE_FORMAT_PCM => SampleType::Int,
-            WAVE_FORMAT_IEEE_FLOAT => SampleType::Float,
-            _ => {
-                return Err(WasapiError::new("Unsupported format").into());
-            }
-        };
-        let storebits = 8 * blockalign / channels;
-        Ok(WaveFormat::new(
-            storebits,
-            validbits,
-            &sample_type,
-            samplerate,
-            channels,
-        ))
-    }
-
-    /// get a pointer of type WAVEFORMATEX, used internally
-    pub fn as_waveformatex_ptr(&self) -> *const WAVEFORMATEX {
-        &self.wave_fmt as *const _ as *const WAVEFORMATEX
-    }
-
-    /// Read nBlockAlign.
-    pub fn get_blockalign(&self) -> u32 {
-        self.wave_fmt.Format.nBlockAlign as u32
-    }
-
-    /// Read nAvgBytesPerSec.
-    pub fn get_avgbytespersec(&self) -> u32 {
-        self.wave_fmt.Format.nAvgBytesPerSec
-    }
-
-    /// Read wBitsPerSample.
-    pub fn get_bitspersample(&self) -> u16 {
-        self.wave_fmt.Format.wBitsPerSample
-    }
-
-    /// Read wValidBitsPerSample.
-    pub fn get_validbitspersample(&self) -> u16 {
-        unsafe { self.wave_fmt.Samples.wValidBitsPerSample }
-    }
-
-    /// Read nSamplesPerSec.
-    pub fn get_samplespersec(&self) -> u32 {
-        self.wave_fmt.Format.nSamplesPerSec
-    }
-
-    /// Read nChannels.
-    pub fn get_nchannels(&self) -> u16 {
-        self.wave_fmt.Format.nChannels
-    }
-
-    /// Read dwChannelMask.
-    pub fn get_dwchannelmask(&self) -> u32 {
-        self.wave_fmt.dwChannelMask
-    }
-
-    /// Read SubFormat.
-    pub fn get_subformat(&self) -> WasapiRes<SampleType> {
-        let subfmt = match self.wave_fmt.SubFormat {
-            KSDATAFORMAT_SUBTYPE_IEEE_FLOAT => SampleType::Float,
-            KSDATAFORMAT_SUBTYPE_PCM => SampleType::Int,
-            _ => {
-                return Err(WasapiError::new(
-                    format!("Unknown subformat {:?}", { self.wave_fmt.SubFormat }).as_str(),
-                )
-                .into());
-            }
-        };
-        Ok(subfmt)
-    }
-}
-
-type OptionBox<T> = Option<Box<T>>;
-
-/// A structure holding the callbacks for notifications
-pub struct EventCallbacks {
-    simple_volume: OptionBox<dyn Fn(f32, bool, Guid)>,
-    channel_volume: OptionBox<dyn Fn(usize, f32, Guid)>,
-    state: OptionBox<dyn Fn(SessionState)>,
-    disconnected: OptionBox<dyn Fn(DisconnectReason)>,
-    iconpath: OptionBox<dyn Fn(String, Guid)>,
-    displayname: OptionBox<dyn Fn(String, Guid)>,
-    groupingparam: OptionBox<dyn Fn(Guid, Guid)>,
-}
-
-impl Default for EventCallbacks {
-    fn default() -> Self {
-        Self::new()
-    }
-}
-
-impl EventCallbacks {
-    /// Create a new EventCallbacks with no callbacks set
-    pub fn new() -> Self {
-        Self {
-            simple_volume: None,
-            channel_volume: None,
-            state: None,
-            disconnected: None,
-            iconpath: None,
-            displayname: None,
-            groupingparam: None,
-        }
-    }
-
-    /// Set a callback for OnSimpleVolumeChanged notifications
-    pub fn set_simple_volume_callback(&mut self, c: impl Fn(f32, bool, Guid) + 'static) {
-        self.simple_volume = Some(Box::new(c));
-    }
-    /// Remove a callback for OnSimpleVolumeChanged notifications
-    pub fn unset_simple_volume_callback(&mut self) {
-        self.simple_volume = None;
-    }
-
-    /// Set a callback for OnChannelVolumeChanged notifications
-    pub fn set_channel_volume_callback(&mut self, c: impl Fn(usize, f32, Guid) + 'static) {
-        self.channel_volume = Some(Box::new(c));
-    }
-    /// Remove a callback for OnChannelVolumeChanged notifications
-    pub fn unset_channel_volume_callback(&mut self) {
-        self.channel_volume = None;
-    }
-
-    /// Set a callback for OnSessionDisconnected notifications
-    pub fn set_disconnected_callback(&mut self, c: impl Fn(DisconnectReason) + 'static) {
-        self.disconnected = Some(Box::new(c));
-    }
-    /// Remove a callback for OnSessionDisconnected notifications
-    pub fn unset_disconnected_callback(&mut self) {
-        self.disconnected = None;
-    }
-
-    /// Set a callback for OnStateChanged notifications
-    pub fn set_state_callback(&mut self, c: impl Fn(SessionState) + 'static) {
-        self.state = Some(Box::new(c));
-    }
-    /// Remove a callback for OnStateChanged notifications
-    pub fn unset_state_callback(&mut self) {
-        self.state = None;
-    }
-
-    /// Set a callback for OnIconPathChanged notifications
-    pub fn set_iconpath_callback(&mut self, c: impl Fn(String, Guid) + 'static) {
-        self.iconpath = Some(Box::new(c));
-    }
-    /// Remove a callback for OnIconPathChanged notifications
-    pub fn unset_iconpath_callback(&mut self) {
-        self.iconpath = None;
-    }
-
-    /// Set a callback for OnDisplayNameChanged notifications
-    pub fn set_displayname_callback(&mut self, c: impl Fn(String, Guid) + 'static) {
-        self.displayname = Some(Box::new(c));
-    }
-    /// Remove a callback for OnDisplayNameChanged notifications
-    pub fn unset_displayname_callback(&mut self) {
-        self.displayname = None;
-    }
-
-    /// Set a callback for OnGroupingParamChanged notifications
-    pub fn set_groupingparam_callback(&mut self, c: impl Fn(Guid, Guid) + 'static) {
-        self.groupingparam = Some(Box::new(c));
-    }
-    /// Remove a callback for OnGroupingParamChanged notifications
-    pub fn unset_groupingparam_callback(&mut self) {
-        self.groupingparam = None;
-    }
-}
-
-/// Reason for session disconnect
-#[derive(Debug)]
-pub enum DisconnectReason {
-    DeviceRemoval,
-    ServerShutdown,
-    FormatChanged,
-    SessionLogoff,
-    SessionDisconnected,
-    ExclusiveModeOverride,
-    Unknown,
-}
-
-/// Wrapper for IAudioSessionEvents
-#[implement(Windows::Win32::Media::Audio::CoreAudio::IAudioSessionEvents)]
-struct AudioSessionEvents {
-    callbacks: Weak<EventCallbacks>,
-}
-
-#[allow(non_snake_case)]
-impl AudioSessionEvents {
-    /// Create a new AudioSessionEvents instance, returned as a IAudioSessionEvent.
-    #[allow(clippy::new_ret_no_self)]
-    pub fn new(callbacks: Weak<EventCallbacks>) -> Self {
-        Self {
-            callbacks,
-        }
-    }
-
-    fn OnStateChanged(&mut self, newstate: AudioSessionState) -> HRESULT {
-        trace!("state change: {:?}", newstate);
-        #[allow(non_upper_case_globals)]
-        let sessionstate = match newstate {
-            AudioSessionStateActive => SessionState::Active,
-            AudioSessionStateInactive => SessionState::Inactive,
-            AudioSessionStateExpired => SessionState::Expired,
-            _ => return S_OK,
-        };
-        if let Some(callbacks) = &mut self.callbacks.upgrade() {
-            if let Some(callback) = &callbacks.state { 
-                callback(sessionstate);
-            }
-        }
-        S_OK
-    }
-
-    fn OnSessionDisconnected(
-        &mut self,
-        disconnectreason: AudioSessionDisconnectReason,
-    ) -> HRESULT {
-        trace!("Disconnected");
-        #[allow(non_upper_case_globals)]
-        let reason = match disconnectreason {
-            DisconnectReasonDeviceRemoval => DisconnectReason::DeviceRemoval,
-            DisconnectReasonServerShutdown => DisconnectReason::ServerShutdown,
-            DisconnectReasonFormatChanged => DisconnectReason::FormatChanged,
-            DisconnectReasonSessionLogoff => DisconnectReason::SessionLogoff,
-            DisconnectReasonSessionDisconnected => DisconnectReason::SessionDisconnected,
-            DisconnectReasonExclusiveModeOverride => DisconnectReason::ExclusiveModeOverride,
-            _ => DisconnectReason::Unknown,
-        };
-
-        if let Some(callbacks) = &mut self.callbacks.upgrade() {
-            if let Some(callback) = &callbacks.disconnected {
-                callback(reason);
-            }
-        }
-        S_OK
-    }
-
-    fn OnDisplayNameChanged(
-        &mut self,
-        newdisplayname: PWSTR,
-        eventcontext: *const ::windows::Guid,
-    ) -> ::windows::HRESULT {
-        let wide_name = unsafe { U16CString::from_ptr_str(newdisplayname.0) };
-        let name = wide_name.to_string_lossy();
-        trace!("New display name: {}", name);
-        if let Some(callbacks) = &mut self.callbacks.upgrade() {
-            if let Some(callback) = &callbacks.displayname {
-                let context = unsafe { *eventcontext };
-                callback(name, context);
-            }
-        }
-        S_OK
-    }
-
-    fn OnIconPathChanged(
-        &mut self,
-        newiconpath: PWSTR,
-        eventcontext: *const ::windows::Guid,
-    ) -> ::windows::HRESULT {
-        let wide_path = unsafe { U16CString::from_ptr_str(newiconpath.0) };
-        let path = wide_path.to_string_lossy();
-        trace!("New icon path: {}", path);
-        if let Some(callbacks) = &mut self.callbacks.upgrade() {
-            if let Some(callback) = &callbacks.iconpath {
-                let context = unsafe { *eventcontext };
-                callback(path, context);
-            }
-        }
-        S_OK
-    }
-
-    fn OnSimpleVolumeChanged(
-        &mut self,
-        newvolume: f32,
-        newmute: BOOL,
-        eventcontext: *const ::windows::Guid,
-    ) -> ::windows::HRESULT {
-        trace!("New volume: {}, mute: {:?}", newvolume, newmute);
-        if let Some(callbacks) = &mut self.callbacks.upgrade() {
-            if let Some(callback) = &callbacks.simple_volume {
-                let context = unsafe { *eventcontext };
-                callback(newvolume, bool::from(newmute), context);
-            }
-        }
-        S_OK
-    }
-
-    fn OnChannelVolumeChanged(
-        &mut self,
-        channelcount: u32,
-        newchannelvolumearray: *mut f32,
-        changedchannel: u32,
-        eventcontext: *const ::windows::Guid,
-    ) -> ::windows::HRESULT {
-        trace!("New channel volume for channel: {}", changedchannel);
-        let volslice =
-            unsafe { slice::from_raw_parts(newchannelvolumearray, channelcount as usize) };
-        let newvol = volslice[changedchannel as usize];
-        if let Some(callbacks) = &mut self.callbacks.upgrade() {
-            if let Some(callback) = &callbacks.channel_volume {
-                let context = unsafe { *eventcontext };
-                callback(changedchannel as usize, newvol, context);
-            }
-        }
-        S_OK
-    }
-
-    fn OnGroupingParamChanged(
-        &mut self,
-        newgroupingparam: *const ::windows::Guid,
-        eventcontext: *const ::windows::Guid,
-    ) -> ::windows::HRESULT {
-        trace!("Grouping changed");
-        if let Some(callbacks) = &mut self.callbacks.upgrade() {
-            if let Some(callback) = &callbacks.groupingparam {
-                let context = unsafe { *eventcontext };
-                let grouping = unsafe { *newgroupingparam };
-                callback(grouping, context);
-            }
-        }
-        S_OK
-    }
-
-}
